/* glplib1b.c */

/*----------------------------------------------------------------------
-- This code is part of GNU Linear Programming Kit (GLPK).
--
-- Copyright (C) 2000, 01, 02, 03, 04, 05, 06, 07 Andrew Makhorin,
-- Department for Applied Informatics, Moscow Aviation Institute,
-- Moscow, Russia. All rights reserved. E-mail: <mao@mai2.rcnet.ru>.
--
-- GLPK is free software; you can redistribute it and/or modify it
-- under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2, or (at your option)
-- any later version.
--
-- GLPK is distributed in the hope that it will be useful, but WITHOUT
-- ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-- or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
-- License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with GLPK; see the file COPYING. If not, write to the Free
-- Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
-- 02110-1301, USA.
----------------------------------------------------------------------*/

#include "glpcfg.h"

#if !defined(GLP_TM_SPEC) || GLP_TM_SPEC == 0

/* platform-independent ISO C version */

#include <stdlib.h>
#include <time.h>
#include "glplib.h"

/*----------------------------------------------------------------------
-- jday - convert calendar date to Julian day.
--
-- This procedure converts a calendar date, Gregorian calendar, to the
-- corresponding Julian day number j. From the given day d, month m, and
-- year y, the Julian day number j is computed without using tables. The
-- procedure is valid for any valid Gregorian calendar date. */

static int jday(int d, int m, int y)
{     int c, ya, j;
      if (m > 2) m -= 3; else m += 9, y--;
      c = y / 100;
      ya = y - 100 * c;
      j = (146097 * c) / 4 + (1461 * ya) / 4 + (153 * m + 2) / 5 + d +
         1721119;
      return j;
}

/*----------------------------------------------------------------------
-- lib_get_time - determine the current universal time.
--
-- *Synopsis*
--
-- #include "glplib.h"
-- double lib_get_time(void);
--
-- *Returns*
--
-- The routine lib_get_time returns the current universal time (UTC),
-- in seconds, elapsed since 12:00:00 GMT January 1, 2000. */

double lib_get_time(void)
{     time_t timer;
      struct tm *tm;
      int j2000, j;
      double secs;
      timer = time(NULL);
      tm = gmtime(&timer);
      j2000 = 2451545 /* = jday(1, 1, 2000) */;
      j = jday(tm->tm_mday, tm->tm_mon + 1, 1900 + tm->tm_year);
      secs = (((double)(j - j2000) * 24.0 + (double)tm->tm_hour) * 60.0
         + (double)tm->tm_min) * 60.0 + (double)tm->tm_sec - 43200.0;
      return secs;
}

#elif GLP_TM_SPEC == 1

/* GNU/Linux version */

#include <sys/time.h>
#include "glplib.h"

double lib_get_time(void)
{     struct timeval tv;
      long int tv_sec0;
      double secs;
      /* tv_sec0 = (12:00:00 GMT 1/1/2000) - (00:00:00 GMT 1/1/1970) */
      tv_sec0 = 946728000;
      gettimeofday(&tv, NULL);
      secs = (double)(tv.tv_sec - tv_sec0) + 1e-6 * (double)tv.tv_usec;
      return secs;
}

#elif GLP_TM_SPEC == 2

/* 32-bit Windows version */

#include <windows.h>
#include "glplib.h"

double lib_get_time(void)
{     SYSTEMTIME st;
      FILETIME ft0, ft;
      double secs;
      /* ft0 = 12:00:00 GMT January 1, 2000 */
      ft0.dwLowDateTime  = 0xBAA22000;
      ft0.dwHighDateTime = 0x01BF544F;
      GetSystemTime(&st);
      SystemTimeToFileTime(&st, &ft);
      /* ft = ft - ft0 */
      if (ft.dwLowDateTime >= ft0.dwLowDateTime)
      {  ft.dwLowDateTime  -= ft0.dwLowDateTime;
         ft.dwHighDateTime -= ft0.dwHighDateTime;
      }
      else
      {  ft.dwLowDateTime  += (0xFFFFFFFF - ft0.dwLowDateTime) + 1;
         ft.dwHighDateTime -= ft0.dwHighDateTime + 1;
      }
      secs = (4294967296.0 * (double)(LONG)ft.dwHighDateTime +
         (double)ft.dwLowDateTime) / 10000000.0;
      return secs;
}

#endif

/* eof */
